//
// dicmain.c

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"

//#define RecNb	32
//#define RecNb	48
#define RecNb	64
//#define WdNbPg	600
#define WdNbPg	700
#define WdDisp	30
#define Size	256

FILE *ejd;
char Dicfile[] = "EJDIC.HON";
char ejr[Size], pg[RecNb][Size];
char wd[100], egw[WdNbPg][WordLen],jpw[WdNbPg][JWordLen],zkw[WdNbPg][ZokuLen];
/*char dics[26][27];*/
char dics[26][27] = {	"bcdefghilmnprstuvwx ","aeeeiloruy          ",
			"aehilooru           ","aeioruy             ",
			"acdefgilmnqrsuvxy   ","aeiloru             ",
			"aehiloruy           ","aaaeiouy            ",
			"cdflmnnnorst        ","aeiou               ",
			"aeino               ","aeiou               ",
			"aaeiouy             ","aeiouy              ",
			"abcdfhiklmnprstuvwxy","aehilnoruy          ",
			"u                   ","aehiou              ",
			"acehiklmnopqtuwy    ","aehhioruwy          ",
			"fglmnprst           ","aeiou               ",
			"aehiory             ","ey                  ",
			"aeiou               ","eio                 " };
int  dic[26] = {0,20,31,41,49,67,75,85,94,107,113,119,125,133,140,161,
	172,174,181,198,209,219,225,233,236,242};
int  wnb;
char jwd[JWordLen], zwd[ZokuLen];

main(){
    int  i, j, jz, ppt, pt0, nx, ia, in, iz, Ans, nxs[RecNb];
    int  wi;
    char tsk, buf[40];
    ejd = fopen(Dicfile, "r+");
    if(ejd == NULL){ dicinit();
      printf("܂\n");	exit(1);
    }
    while((char)1){
      wdGet();	if(strcmp(wd, "END")==0) break;
      if(strcmp(wd, "FIND") == 0){ find();	continue; }
      if(strcmp(wd, "COPY") == 0){ copy();	continue; }
      ppt = ptcal(dics, dic);
      getpage(ppt, &nx);	/* e */
      j = 0;	pt0 = ppt;	strcpy(pg[j], ejr);
      nxs[j] = nx;
      while(nx > 0 && nx < MaxNr){
	j++;
	if(j >= RecNb-3){
	  printf("\n PR[hƂĖtɂȂ肩Ă܂I");
	  for(i = j-3; i <= j; i++)
	    printf("pg[%d] = %s \n", i, pg[i]);
	  if(j >= RecNb){
	    printf("PR[hƂĖtłI \n");
	    exit(1);
	  }
	}
	ppt = nx;
	getpage(ppt, &nx);	strcpy(pg[j], ejr);	nxs[j] = nx;
      }
      jz =j;
      pageDecompose(pg, j, &wnb);	/* Pɕ */
      ia = wnb;
      for(i = 0; i <= wnb; i++)
	if(strncmp(egw[i], wd, strlen(wd))==0){ ia = i;
	  if(ia > 0) ia--;
	  break;
	}
      Ans = '\0';
      while(Ans != '6'){
	iz = ia+WdDisp;
	if(iz > wnb) iz = wnb;
	if(ia == -1) printf("(-1)\n");
	else{
	  for(i = 0; i <= iz; i++){
	    printf("(%d)", i);	color_red;	printf("%s", egw[i]);
	    color_white;	printf(";%s;%s;    ", jpw[i],zkw[i]);
	  }
	  printf("\n");
          wi = 0;
       	  for(i = 0; i <= iz; i++)
	    if(strcmp(wd, egw[i]) == 0){
	      printf("\n(%d)", i);	color_red; printf("%s", egw[i]);
	      color_white;	printf(";%s;%s; ", jpw[i],zkw[i]);
              wi++;
	    }
	  printf("\n");
          if(wi >= 8) printf("wi = %d ȏo^ł܂\n", wi);
        }
	Ans = '\0';
	printf("\n1 C  2 o^  3 폜  4 y[W̎̍s \n");
	printf("5 y[W̑O̍s  6 ̃y[W͏I\n");
	while(Ans < '1' || Ans > '6'){ //Ans = getchar();
	  //gets(buf);
          getline(buf, 10);
          if(strlen(buf) == 1) Ans = *buf;
        }
	if(Ans <= '3'){
	  in = -1;
	  if(Ans == '2') goto MAIN1;
	  puts("Cio^A폜j̒Pinj͉ԂłB");
	  while(in < 0 || (in > wnb && Ans != '2') || in > wnb+1){
	    //gets(buf);
            scanf("%s", buf);
            if(! isdigit(*buf)) continue;
	    in = atoi(buf);
	  }
	  if(Ans == '3'){
	    for(i = in; i <= wnb; i++){
	      strcpy(egw[i], egw[i+1]);
	      strcpy(jpw[i], jpw[i+1]);
	      strcpy(zkw[i], zkw[i+1]);
	    }
	    wnb--;
	    if(iz > wnb) iz = wnb;
	    continue;
	  }
	  MAIN1:
	  if(Ans == '2'){
	    in = getno();
	    if(wnb >= WdNbPg){
	      printf("Too many words in one page! egw[wnb] = %s \n", egw[wnb]);
	      exit(1);
	    }
	    if(in <= wnb)
	      for(i = wnb; i >= in; i--){
		strcpy(egw[i+1], egw[i]);
		strcpy(jpw[i+1], jpw[i]);
		strcpy(zkw[i+1], zkw[i]);
	      }
	    wnb++;	strcpy(egw[in], wd);	tsk = '5';
	  }
	  if(Ans == '1'){
	    tsk = '0';
	    printf("ӏ́@@@");
	    printf("1 p@@@");
	    printf("2 @@@");
	    printf("3 i@@@4 @@@5 S@@@\n");
/*	    while(tsk < '1' || tsk > '5') tsk = getchar();
	    gets(buf);*/
	    while(tsk < '1' || tsk > '5'){
              gets(buf);
              //scanf("%s", buf);
              tsk = buf[0]; }
	  }
	  shuusei(in, tsk);	continue;
	}
	if(Ans == '4'){
	  ia += WdDisp+1;	iz = ia+WdDisp;
	  if(iz > wnb) iz = wnb;
	}
	if(Ans == '5'){
	  ia -= WdDisp+1;
	  if(ia < 0) ia = 0;
	  iz = ia+WdDisp;
	  if(iz > wnb) iz = wnb;
	}
	if(Ans == '6') pageWrite(wnb, &jz, nxs, pt0);
      }
    }
    fclose(ejd);
}

ptcal(dics, dic)
char dics[26][27];
int  dic[26];
{
    int  i, p0, p1, ppt;
    char wd3;
    p0 = wd[0];
    if(p0 < 92) ppt = p0-65;
    else ppt = p0-97;
    if(ppt < 0 || ppt > 25){ printf("Character error!\n");	exit(1); }
    if(strlen(wd) == 1){ i = 0;	goto PTA; }
    p1 = wd[1];
    if(p1 < 65){ i = 0;	goto PTA; }
    if(p1 < 92) p1 += 32;
    i = 0;
    while(dics[ppt][i] != p1){
      if(dics[ppt][i] > p1){ i--;	break; }
      if(dics[ppt][i] == ' ' || dics[ppt][i] == '\0'){
	i = 0;	goto PTA;
      }
      else i++;
    }
    i++;
    PTA:
    ppt = dic[ppt]+i+1;
    if(ppt == 23){
      if(wd[2] == '\0' || wd[2] == ' '){
/*	if(wd[2] == ' ') wd3 = wd[3];
	else{
          ppt += 2;
          return(ppt);
        }*/
/*	if((wd3 >= 'o' && wd3 <= 'z') || (wd3 >= 'O' && wd3 <= 'Z')) ppt += 2;
	else ppt++;*/
	ppt++;
      }
      return(ppt);
    }
    if(ppt == 38 && wd[2] >= 'n'){ ppt++;	return(ppt); }
    if(ppt == 87 && wd[2] >= 'v'){
      if(strstr(wd, "have a ") || strstr(wd, "Have a ")) ppt++;
      else ppt += 2;
      return(ppt);
    }
    if(ppt == 101){
      if(wd[2] != '\0' && wd[2] != ' '){ ppt += 2;	return(ppt); }
      else{
	if(wd[2]) wd3 = tolower(wd[3]);
	else wd3 = '\0';
	if(wd3 > 'o'){ ppt++;	return(ppt); }
      }
    }
    if(ppt == 127 && wd[2] >= 'l'){ ppt++;	return(ppt); }
    if(ppt == 202 && wd[2] >= 'i') ppt++;
    return(ppt);
}

getpage(ppt, nx)
int  ppt, *nx;
{
    int  len;
    long ptl;
    char *ptc, yn;
    ptl = (long)(ppt-1)* Size;
    rewind(ejd);
    if(fseek(ejd, ptl, 0)){
      printf("seek error\n");	exit(1);
    }
    fread(ejr, Size-4,1, ejd);
    fscanf(ejd, "%4d", nx);
    *(ejr+ Size-4) = '\0';
    if(ptc = (char*)rindex(ejr, ':')) *(ptc+1) = '\0';
    else{
      if(*ejr && ejr != (char*)strstr(ejr, "                           ")){
	printf("Abnomal   ejr = %s \n", ejr);
	exit(1);
      }
    }
    if(*nx != 0 && (*nx < 246 || *nx > MaxNr)){
      len = strlen(ejr);
      printf("nx error ppt = %d \n", ppt);
      printf("Continue, or Abort ?  [C/A] \n");
      yn = '\0';
      while(tolower(yn) != 'c' && tolower(yn) != 'a') yn = getchar();
      if(tolower(yn) == 'a') exit(1);
    }
}

pageDecompose(pg, jz, wnb)
int  jz, *wnb;
char pg[RecNb][Size];
{
    int  i, j, pc, pe, pj, ln;
    char rd[Size], r1[150];
    i = -1;
    for(j = 0; j <= jz; j++){
      strcpy(rd, pg[j]);
      while(pc = instrc(rd, ':')){
	if(pc >= 130) printf("Too long rd = %s strlen(r1) = %d \n",rd,pc);
	left(r1, rd, pc-1);
	while(r1[0] == ' ') cutoff(r1, 0, 1);
	pe = instrc(r1, ';');
	if(pe <= 1 || r1[0] == '\0') goto NXT;
	if(pe >= WordLen){
	  printf("Too long %s \n", r1);
	  exit(1);
	}
	i++;
	left(egw[i], r1, pe-1);
	ln = strlen(r1);
	if(pj = ninstrc(pe+1, r1, ';')){
	  if(pj-pe-1 >= JWordLen){
	  }
	  if(ln-pj >= ZokuLen){
	  }
	  mid(r1, pe+1, pj-pe-1, jpw[i]);
	  right(r1, zkw[i], ln-pj);
	}
	else{
	  if(ln -pe >= JWordLen){
	  }
	  right(r1, jpw[i], ln-pj);	zkw[i][0] = '\0';
	}
	if(i >= WdNbPg-12){
	  printf("L[̒PꂪtɂȂ肩Ă܂\n");
	  printf("limit = WdNbPg  egw[%d] = %s \n", i, egw[i]);
	  exit(1);
	}
	NXT:	ptcut(rd, pc);
      }
    }
    *wnb = i;
}

endOfRec(fp, recLen)
int  recLen;
FILE *fp;
{
    long ptl, nl;
    int n;
    rewind(fp);
    fseek(fp, -1L, 2);
    ptl = ftell(fp);
    nl = ptl / recLen;
    n = (int)nl+1;
    return(n);
}

pageWrite(wnb, jz,nxs, ppt)
int  wnb, *jz, ppt, nxs[RecNb];
{
    int  i, j, pt0, lnr;
    long ptl;
    char rcd[Size];
    j = 0;
    pt0 = ppt;
    ptl = (long)(ppt-1) * Size;
    rcd[0] = '\0';
    for(i = 0; i <= wnb; i++){
      if(strlen(rcd)+strlen(egw[i])+strlen(jpw[i])+strlen(zkw[i]) >= Size-7){
	strcpy(pg[j], rcd);	rcd[0] = '\0';
	if(j == *jz) nxs[j] = endOfRec(ejd, Size)+1;
	if(j > *jz) nxs[j] = nxs[j-1]+1;
	j++;
      }
      while(egw[i][0] == ' ') cutoff(egw[i], 0, 1);
      if(egw[i][0] == '\0') continue;
      strink(rcd, egw[i], ";", jpw[i], ";", zkw[i], ":", (char*)0);
      lnr = strlen(rcd);
      if(lnr >= Size-4){
	printf("Too long lnr = %d  rcd = %s \n", lnr,rcd);
	exit(1);
      }
    }
    strcpy(pg[j], rcd);
    nxs[j] = 0;	*jz = j;
    for(i = 0; i <= *jz; i++){
      rewind(ejd);
      ptl = (long)(ppt-1)* Size;
      if(fseek(ejd, ptl, 0)){
	printf("dictionary seek error \n");	exit(1);
      }
      fprintf(ejd, "%-252s%04d", pg[i], nxs[i]);
      ppt = nxs[i];
    }
    if(j > 0) ppt = nxs[j-1];
    else ppt = pt0;
    printf("pg[%d] = %s  ppt = %d \n", j, pg[j], ppt);
    ppt = endOfRec(ejd, Size);
    printf("end_of_rec = %d \n", ppt);
}

getZoku(zkn)
char zkn[];
{
    char tail;
    int  ct;
    puts("\n͂ĉ ïꗗ̏ꍇ \\ j");
   getZ1:
    printf(" = ");
    zkn[0] = '3';
    ct = 0;
    while(!isalpha(*zkn)){
       gets(zkn);
	ct++;
/*       if(zkn[0] == '\\'){ cat "ZOKU.HLP";	ct = 0;	break; }*/
       if(ct == 2 || zkn[0] == 13) break;
    }
    if(ct == 0) goto getZ1;
}

shuusei(in, tsk)
int in;
char tsk;
{
    char cd, yn, buf[50];
    switch(tsk){
      case '1':
      egw[in][0] = '\0';
      puts("nɂ́uit is # that >vƂЂ傤g܂B");
      printf("p = ");
      while(!isalpha(egw[in][0]))
        //gets(egw[in]);
        scanf("%s", egw[in]);
      if(tsk == '1') return;
      case '5':;
      case '2':
      jpw[in][0] = '\0';
      puts("ɂ́Áu#vƁu>v̑Ɂu$v(Ëׂ)Ɓu<v(ׂ̈)g܂B");
      printf(" = ");
//	gets(jpw[in]);
      getline(jpw[in], JWordLen);
      if(jpw[in][0] == '\0'){
	yn = '\0';
	printf("͋ijł悢łH@[y/n]\n");
	while(tolower(yn) != 'y' && tolower(yn) != 'n'){
          //yn = getchar();
          //gets(buf);
          getline(buf, 10);
          yn = *buf;
        }
        if(tolower(yn) == 'n'){
	  while(jpw[in][0] == '\0'){
	    printf("͂ĉ\n = ");	gets(jpw[in]);
	  }
	}
      }
      case '3':
      puts("\n̒P̕iR[h̔ԍ̒IŉB\n");
      puts("1   2 㖼  3   4   5 `e");
      puts("6   7 Ou  8   9 ڑ  X ԓ");
      cd = '\0';	printf("i = ");
      while((cd < '1' || cd > '9') && cd != 'X') cd = getchar();
      if(tsk == '3'){ jpw[in][0] = cd;	return; }
      else insertc(jpw[in], 0, cd, strlen(jpw[in]));
      if(tsk == '2') return;
      case '4':
      getZoku(zkw[in]);
    }
}

wdGet(){
    puts("\nI鎞́AEND Ƒ啶Ń^CvĉB");
    wd[0] = '\0';
    puts("\no^iCjpPinj ? ");
    printf("p = ");
/*    while(! isalpha(wd[0])) gets(wd);*/
    while(wd[0] == '\0' || (unsigned char)wd[0] > 128) gets(wd);
}

dicinit(){
    int  i, nx;
    long ptl;
    char rec[Size];
    ejd = fopen(Dicfile, "w+");
    nx = 0;	*rec = '\0';
    for(i = 0; i <= 245; i++){
      ptl = (long)i * Size;
      if(fseek(ejd, ptl, 0)){
	printf("Seek error\n");	exit(1);
      }
      fprintf(ejd, "%-252s%04d", rec, nx);
    }
    fclose(ejd);
}


isVowel(c)
char c;
{
    int  j = 0;
    if(c == 'a' || c == 'e' || c == 'i' || c == 'o' || c == 'u') j = 1;
    if(c == 'A' || c == 'E' || c == 'I' || c == 'O' || c == 'U') j = 1;
    return(j);
}

ptcpy(c, pt1, pt2)
char c[], *pt1, *pt2;
{
     char *pt;
     pt = pt1;
     while(pt != pt2) *(c++) = *(pt++);
    *c = '\0';
}

wdcpy(st1, st2)
char *st1, *st2;
{
    while(*st2 && (*st2 != ' ')) *st1++ = *st2++;
    *st1 = '\0';
}

cutoff(a, x, i)
char *a;
int  x, i;
{
    int  ln;
    char *b;
    ln = strlen(a);
    if(x < ln){
      b = a+x+i;	*(a+x) = '\0';
      if(ln > x+1) strcat(a, b);
    }
}

ptcut(pt, i)
char *pt;
int  i;
{
    char *pta, *ptq;
    int  ln;
    ln = strlen(pt);
    if(i >= ln) *pt = '\0';
    else{
      pta = pt;	ptq = pt + i;
      do{
	*(pta++) = *(ptq++);
      } while(*ptq);
      *pta = '\0';
    }
}

ptptcut(pt1, pt2)
char *pt1, *pt2;
{
    char *pta = pt1, *ptz = pt2;
    do{
      *(pta++) = *(++ptz);
    }	while(*ptz);
}

insertc(a, i, c, ln)
char a[], c;
int  i, ln;
{
    int  j;
    if(i < ln)
      for(j = ln; j > i; j--) a[j] = a[j-1];
    a[i] = c;	a[ln+1] = '\0';
}

ptinsert(pt, s)
char *pt, s[];
{
    char dums[200];
    strcpy(dums, pt);	strcpy(pt, s);	strcat(pt, dums);
}

left(a, c, i)
char a[], c[];
int  i;
{
    int  x;
    if(i > strlen(c)) i = strlen(c);
    for(x = 0; x <= i-1; x++) a[x] = c[x];
    a[i] = '\0';
}

replaces(a, k, ln2, c)
char a[], c[];
int  k, ln2;
{
    char *bpt, dumc[256];
    bpt = a + k + ln2;
    strcpy(dumc, c);	strcat(dumc, bpt);
    *(a+k) = '\0';	strcat(a, dumc);
}

ptreplace(pt, ln, c)
char c[], *pt;
int  ln;
{
    char *pt2 = pt+ln, dums[200];
    strcpy(dums, pt2);	*pt = '\0';
    strink(pt, c, dums, (char*)0);
}

strink(s1, s2)
char *s1, *s2;
{
    char *p = s1, *q, **a = &s2;
    while(*p) ++p;
    while(q = *(a++)) while(*q) *p++ = *q++;
    *p = '\0';
}

right(c, b, i)
unsigned char *c, *b;
int  i;
{
    int  d;
    unsigned char *pt;
    if(i >= 0){
      d = strlen(c)-i;
      if(d >= 0) pt = c+d;	else pt = c;
      strcpy(b, pt);
    }
    else *b = '\0';
}

mid(a, i, j, b)
int  i, j;
char a[], b[];
{
    char c[160];
    left(c, a, i+j-1);	right(c, b, j);
}

instr(a, b)
char *a, *b;
{
    int  j = 1;
    char *pt;
    if(pt = (char *)strstr(a, b)){
      while( a != pt--) j++;
      return(j);
    }
    else return(0);
}

ninstr(n, a, b)
int  n;
char *a, *b;
{
    int  inst;
    char *pta;
    pta = a+n-1;
    if(inst = instr(pta, b)) return(n+inst-1);
    else return(0);
}

instrc(a, b)
char *a, b;
{
    int  j = 1;
    char *pt;
    if(pt = (char *)index(a, b)){
      while(a != pt--) j++;
      return(j);
    }
    else return(0);
}

ninstrc(n, a, b)
int  n;
char *a, b;
{
    int  inst;
    char *pta;
    pta = a + n - 1;
    if(inst = instrc(pta, b)) return(n+inst-1);
    else return(0);
}

swap(a, b)
char a[], b[];
{
    char c[200];
    strcpy(c, a);	strcpy(a, b);	strcpy(b, c);
}

val(nstr)
char nstr[];
{
    int  ln, i, keta = 1, n = 0;
    ln = strlen(nstr);
    for(i = ln-1; i >= 0; i--){
      n += (keta * (nstr[i]-'0'));	keta *= 10;
    }
    return(n);
}

zentoint(zenkaku)
char zenkaku[];
{
    int  i, ln, numb = 0, keta = 1;
    ln = strlen(zenkaku)/2-1;
    for(i = ln; i >= 0; i--){
      numb += keta*(zenkaku[i*2+1]-79);	keta *= 10;
    }
    return(numb);
}

nhantozen(hankaku,zenkaku)
char hankaku[], zenkaku[];
{
    int  i, ln;
    ln = strlen(hankaku)-1;
    for(i = 0; i <= ln; i++){
      if(hankaku[i] == ','){
	zenkaku[2*i] = '\0';	strcat(zenkaku, "C");
      }
      else{
	zenkaku[2*i] = -126;
	if(hankaku[1] < 97) zenkaku[2*i+1] = hankaku[i]+31;
	else zenkaku[2*i+1] = hankaku[i]-225;
      }
    }
    zenkaku[2*i] = '\0';
}

numbtohan(numb, hankaku)
int  numb;
char hankaku[];
{
    int  i, dig, ln;
    char invdigits[9];
    for(i = 0; ; i++){
      dig = numb % 10;
      invdigits[i] = dig +'0';
      numb -= dig;
      if(numb == 0) break;
      else numb /= 10;
    }
    ln = i;
    invdigits[ln+1] = '\0';
    for(i = 0; i <= ln; i++) hankaku[i] = invdigits[ln-i];
    hankaku[ln+1] = '\0';
}
/*
strstr(str1, str2)
unsigned char *str1, *str2;
{
    char *str;
    str = str1;
    STRST:
    if(strlen(str) < strlen(str2)) return(NULL);
	str = index(str, *str2);
    if(str == '\0') return(NULL);
    if(strncmp(str, str2, strlen(str2))==0) return(str);
    str++;	goto STRST;
}
*/
/*------- predit ------------------------------------------*/

predit(wdh)
char wdh[];
{
    int  i, j, ln;
    char *pt;
    ln = strlen(wdh);
    j = ln;
    for(i = ln-1; i > 0; i--){
      if(wdh[i] == '!' || wdh[i] == '?' || wdh[i] == ';'){
	insertc(wdh, i+1, ' ', j++);
	insertc(wdh, i, ' ', j++);
      }
    }
    ln = strlen(wdh);   j = ln;
    for(i = ln-2; i > 0; i--){
      if(wdh[i] == ',' && i >= ln-4){
	insertc(wdh, i+1, ' ', j++);
	insertc(wdh, i, ' ', j++);
      }
      if(wdh[i] == ',' && i < ln-4)
	if(isdigit(wdh[i+1])==0 ||isdigit(wdh[i+2])==0 ||isdigit(wdh[i+2])==0){
	  insertc(wdh, i+1, ' ', j++);
	  insertc(wdh, i, ' ', j++);
	}
    }
    ln = strlen(wdh);
    j = ln;
    for(i = ln-1; i > 0; i--){
      if(wdh[i] == '.'){
	if(wdh[i-1] != '.' && wdh[i-1] != ' ') insertc(wdh, i, ' ', j++);
	break;
      }
      if(wdh[i] != ' ' && wdh[i] != '.') break;
      if(wdh[i] == ' '){ wdh[i] = '\0';	j--; }
    }

    if(pt = (char*)index(wdh, '(')){
    }
}

find()
{
    int  i, j, ppt, nx, wnb, ia, iz;
    while((char)1){
      puts("\nPinj̃^Cv@IƂ END łB");
      printf("p = ");	wd[0] = '\0';
      while(! isalpha(wd[0])) gets(wd);
      if(strcmp(wd, "END")==0) break;
      ppt = ptcal(dics, dic);
      getpage(ppt, &nx);
 printf("--- find --- ppt = %d  nx = %d  ejr = %s \n", ppt,nx, ejr);
      j = 0;
      strcpy(pg[j], ejr);
      pageDecompose(pg, j, &wnb);
      ia = wnb;
      for(i = 0; i <= wnb; i++)
	if(strcmp(egw[i], wd) == 0){ ia = i;	break; }
      iz = wnb;
      if(ia == -1);
      else
	for(i = ia; i <= iz; i++){
	  if(strcmp(egw[i], wd) != 0) break;
	  printf("(%4d)  %-20s %-20s %-20s \n", i, egw[i], jpw[i], zkw[i]);
	}
    }
}

// ̓t@C̃f[^̏ pP:XP:: X͕iԍ
copy()
{
    FILE *fp;
    int  i, j, jz, ppt, pt0, nx, ia, in, iz, nxs[RecNb], eq, ln, re;
    char file[32], line[100], wd1[WordLen];
    char *p0, *pt1, *pt2;
    puts("\n͂t@C(ex. WDREG)̃^Cv@IƂ END łB");
    printf("̓t@C = ");	file[0] = '\0';
    while(! isalpha(file[0])) gets(file);
    if(strcmp(file, "END")==0) return;
    fp = fopen(file, "r");
    while((char)1){
      if(fgets(line, 90, fp) == NULL) break;
      if(!isalpha(line[0])) continue;
      if((p0 = (char*)index(line, ':')) ==0) continue;
      ptcpy(wd, line, p0);
      pt1 = (char*)index(p0+1, ':');	ptcpy(jwd, p0+1, pt1);
      pt2 = (char*)index(pt1+1, ':');	ptcpy(zwd, pt1+1, pt2);
      //      ptcpy(jwd+1, pt2+1, line+strlen(line)-1);
      color_red;	printf("%-20s ", wd);	color_white;
      printf("%-20s %-20s \n", jwd, zwd);
      ppt = ptcal(dics, dic);	re = 0;
      pt0 = ppt;
      COPY1:	j = 0;
      getpage(ppt, &nx);
 printf("--- copy --- ppt = %d  nx = %d \n", ppt,nx);
      strcpy(pg[j], ejr);	nxs[j] = nx;
      while(nx >0 && nx < MaxNr){
	j++;	ppt = nx;	getpage(ppt, &nx);
	strcpy(pg[j], ejr);	nxs[j] = nx;
      }
      jz = j;
      pageDecompose(pg, j, &wnb);
      ia = wnb;
      for(i = 0; i <= wnb; i++)
	if(strcmp(egw[i], wd) == 0){ ia = i;	break; }
      iz = ia+WdDisp;	if(iz > wnb) iz = wnb;
      if(ia == -1) in = 0;
      else{ eq = 1;	strcpy(wd1, wd);
	if(*wd1 < 90) *wd1 += 32;
	for(i = 0; i <= iz; i++){
	  if(strlen(egw[i]) < strlen(wd1)){ ln = strlen(egw[i]);
	    if(strncmp(wd1, egw[i], ln) == 0 &&
	    wd1[ln] == ' '){ eq = 2;	break; }
	  }
	  else ln = strlen(wd1);
	  if(strncmp(egw[i], wd1, ln) > 0)
	    if(!index(wd1,'#') && egw[i][ln] != ' '){ eq = 2;	 break; }
	  if(strcmp(jwd, jpw[i]) == 0 && strcmp(zwd, zkw[i]) == 0)
	    if(strcmp(egw[i], wd) == 0){ eq = 0;	break; }
	}
	if(eq != 0){
	  if(i < 0) i = 0;
	  in = i;
 printf("--- copy --- egw[%d] = %s  in = %d \n", i-1, egw[i-1], in);
	  for(i = wnb; i >= in; i--){
	    strcpy(egw[i+1], egw[i]);
	    strcpy(jpw[i+1], jpw[i]);
	    strcpy(zkw[i+1], zkw[i]);
	  }
	}
      }
      if(ia == -1 || eq != 0){
	wnb++;
	strcpy(egw[in], wd);	strcpy(jpw[in], jwd);	strcpy(zkw[in], zwd);
	pageWrite(wnb, &jz, nxs, pt0);
      }
      if(re == 0 && (strcmp(wd, "be") == 0 || strcmp(wd, "in") == 0)){
	ppt = ++pt0;	re++;	goto COPY1;
      }
      if(re == 0 && strcmp(wd, "have") == 0){
	ppt = --pt0;	re++;	goto COPY1;
      }
    }
    fclose(fp);
}

getno(){
    int  i, ia, iz, in, eq, ln;
    char wd1[WordLen];
      ia = wnb;
      for(i = 0; i <= wnb; i++)
	if(strcmp(egw[i], wd) == 0){ ia = i;	break; }
      iz = ia+WdDisp;	if(iz > wnb) iz = wnb;
      if(ia == -1) in = 0;
      else{ eq = 1;	strcpy(wd1, wd);
	if(*wd1 < 90) *wd1 += 32;
	for(i = 0; i <= iz; i++){
	  if(strlen(egw[i]) < strlen(wd1)){ ln = strlen(egw[i]);
	    if(strncmp(wd1, egw[i], ln) == 0 &&
	    wd1[ln] == ' '){ eq = 2;	break; }
	  }
	  else ln = strlen(wd1);
	  if(strncmp(egw[i], wd1, ln) > 0)
	    if(!index(wd1,'#') && egw[i][ln] != ' '){ eq = 2;	 break; }
	  if(strcmp(jwd, jpw[i]) == 0 && strcmp(zwd, zkw[i]) == 0)
	    if(strcmp(egw[i], wd) == 0){ eq = 0;	break; }
	}
	if(eq != 0){
	  if(i < 0) i = 0;
	  in = i;
 printf("--- getno --- egw[%d] = %s  in = %d \n", in-1, egw[in-1], in);
	}
      }
    return(in);
}

int getline(char *ptr, int maxlen)
{
    int  n;
    char c;
    for(n = 1 ;n < maxlen ;n++ ){
      c = getchar();
      if(c == '\n') break;
      *ptr++ = c;
    }
    *ptr = '\0';
    return(n);
  }
